/**
* This sample shows how to use an analog input sequence acquisition using software and/or hardware trigger
* In this sample we do not use DMA but you can also change it to use DMA
**/
#include "stdafx.h"

/**
* Pointer that will be allocated and that will enables to read the data of the board
**/
USHORT * data = NULL;
DWORD	dataSize = 0;
int	timerIntCounter = 1;
bool isSeqRunning = false;
volatile int autoTrigger = 0;
volatile int errorOccured = 0;			// todo if errorOccured, stop auto trigger


static void StopSchnellTest(HANDLE m_hBoard)
{
	//StopADInTriggerTimer
	//i_PCIe3121_StopAnalogAcquisition ( m_hBoard );
	//StopAnalogInputSequence
	//i_PCIe3121_StopTimer( m_hBoard, 0);
}

/**
* Interrupt routine that enables to get the data read by the board
**/
static void SequenceInterruptRoutine (HANDLE	deviceHandle,
									  BYTE		fifoOverflow,
									  DWORD	interruptSource,
									  DWORD	interruptMask,
									  PWORD	args,
									  DWORD	argsCount )
{
	UNREFERENCED_PARAMETER(args);
	/* switch through the possible interrupt sources */

	int returnValue = 0;

	//printf("interrupt routine enter");
	//_getch();
	/* check the interrupt fifo overflow */
	if ( fifoOverflow )
	{
		printf("Error: interrupt fifo overflow. That means that your software do not handle the interrupts fast enough.\n");
		errorOccured = 1;
	}

	/* DMA */
	if ( interruptSource & 0x4 )
	{
		printf("interruptSource DMA, argsCount = %u \n", argsCount);
		//StopSchnellTest( deviceHandle );
		if ( interruptMask & 0x4 )
		{
			printf("internal sequence fifo overflow \n");
			errorOccured = 4;
		}
		else
		{
			LONG read = 0;
			/* get the sequence data */
			read = l_PCIe3121_ReadSequenceData( deviceHandle, dataSize, data );
			if ( read < 0 )
			{
				printf("error %d in l_PCIe3121_ReadSequenceData \n", (int) read);
				errorOccured = 5;
			}
			else
			{
				DWORD i = 0;
				/* display the values */
				for( i = 0 ; i < argsCount ; i++ )
				{
					/* each 9 values, go to the line */
					if ( i % 9 == 0 )
					{
						printf("\n");
					}
					printf("0x%.4x ", data[i] );
				}
				printf("\n");
				if (!autoTrigger)
					printf("Press space to trigger an acquisition and 'ESC' to exit the acquisition. \n");
			}
			isSeqRunning = false;
		}
	}

	/* End of DMA acquisition */
	if ( interruptSource & 0x8 )
	{
		printf("End of DMA acquisition.\n");
	}

	/* PLD Fifo overflow */
	if ( interruptSource & 0x10 )
	{
		printf("Error: PLD Fifo overflow. This stop the acquisition.\n");
		printf("InterruptSource = 0x%.4x \n", interruptSource);
		errorOccured = 10;
	}

	//====================================================================
	// Timer interrupt
	//====================================================================
	if(interruptSource == 0x02)
	{
		printf("Timer interrupt %d --> Start trigger\n", timerIntCounter);

		returnValue = i_PCIe3121_TriggerAnalogAcquisition( deviceHandle );
		if ( returnValue != 0 )
		{
			printf("Error %d in i_PCIe3121_TriggerAnalogAcquisition \n", returnValue);
			i_PCIe3121_StopAnalogAcquisition( deviceHandle );
			i_PCIe3121_ResetBoardIntRoutine( deviceHandle );
			i_PCIe3121_CloseBoard(deviceHandle);
			_getch();
			return;
		}
		timerIntCounter++;
	}
}

/* entry point */
int _tmain( int /*argc*/, _TCHAR* /*argv[]*/ )
{
	INT returnValue = 0;
	HANDLE boardHandle = NULL;
	ULONG readValue = 0;
	ULONG sequenceNbr = 0;
	BYTE channels[16] = {0};
	BYTE polarity[16] = {0};
	BYTE gain[16] = {0};
	BYTE inputMode = 0;
	BYTE hardwareTrigger = 0;
	BYTE softwareTrigger = 0;
	DWORD sequenceBeforeInterrupt = 0;
	DWORD sequenceTriggerCount = 0;
	int hitChar; 

	DWORD
		dwDelayTime_us = 5,
		dwConvertingTime_us,
		dwSequenceCount = 0,
		dwSequenceBeforeInterrupt;

	sequenceBeforeInterrupt = dwSequenceBeforeInterrupt;
	sequenceNbr = dwSequenceCount;

	/* initialize the acquisition */
	channels[0] = 0;
	channels[1] = 1;
	channels[2] = 2;
	channels[3] = 3;
	channels[4] = 4;
	channels[5] = 5;
	channels[6] = 6;
	channels[7] = 7;
	channels[8] = 8;

	polarity[0] = PCIE3121_OUTPUT_MODE_BIPOLAR;
	polarity[1] = PCIE3121_OUTPUT_MODE_BIPOLAR;
	polarity[2] = PCIE3121_OUTPUT_MODE_BIPOLAR;
	polarity[3] = PCIE3121_OUTPUT_MODE_BIPOLAR;
	polarity[4] = polarity[5] = polarity[6] = polarity[7] = polarity[8] = PCIE3121_OUTPUT_MODE_BIPOLAR;

	gain[0] = PCIE3121_GAIN_1;
	gain[1] = PCIE3121_GAIN_1;
	gain[2] = PCIE3121_GAIN_1;
	gain[3] = PCIE3121_GAIN_1;
	gain[4] = gain[5] = gain[6] = gain[7] = gain[8] = PCIE3121_GAIN_1;

	/* open a board */
	returnValue = OpenBoard(&boardHandle);
	if (returnValue != 0)
	{
		printf("Error %d opening the board \n", returnValue);
		_getch();
		return -1;
	}

	/* display some useful informations */
	printf("\n");
	printf("* * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *\n");
	printf("* This sample will initialize a triggered acquisition\n");
	printf("* The sequence is : \n");
	printf("*   - Channel [8..0], bipolar, gain 1 \n");
	printf("* Delay time is set to %d us \n", dwDelayTime_us);
	printf("* * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *\n\n");

	printf("Converting time you want (in us): ");
	scanf("%d", &dwConvertingTime_us);

	printf("Nombre sequence avant interruption: ");
	scanf("%d", &sequenceBeforeInterrupt);

	/* ask the input mode */
	ReadUlong("Select the input mode (0: Single-ended, 1: Differential) ", 0, 1, &readValue);
	switch ( readValue )
	{
	case 0:
		inputMode = PCIE3121_INPUT_MODE_SINGLE_ENDED;
		break;
	case 1:
		inputMode = PCIE3121_INPUT_MODE_DIFFERENTIAL;
		break;
	}

	/* set the selected input mode */
	returnValue = i_PCIe3121_SetAnalogInputMode( boardHandle, inputMode );
	if (returnValue != 0)
	{
		printf("Error %d in i_PCIe3121_SetAnalogInputMode \n", returnValue);
		goto end;
	}

	/* set the board interrupt routine */
	returnValue = i_PCIe3121_ReleaseSoftwareTriggerWaitingRead( boardHandle );
	if (returnValue != 0)
	{
		printf("Error %d in i_PCIe3121_ReleaseSoftwareTriggerWaitingRead \n", returnValue);
		goto end;
	}

	i_PCIe3121_StopTimer( boardHandle, 0);

	/* set the board interrupt routine */
	returnValue = i_PCIe3121_SetBoardIntRoutine( boardHandle, SequenceInterruptRoutine );
	if (returnValue != 0)
	{
		printf("Error %d in i_PCIe3121_SetBoardIntRoutine \n", returnValue);
		goto end;
	}

	printf("PCIe3121_CONTINUOUS cycle has been set.\n" );

	/* ask the number of sequences before interrupt */
	//ReadUlong("How many sequences do you want before the board generate an interrupt ? ", 1, 0xFFFFFFFF, &sequenceBeforeInterrupt );
	printf("sequence before interrup = %u.\n", sequenceBeforeInterrupt);
	/* allocate memory */
	dataSize = sequenceBeforeInterrupt * 9 * sizeof(USHORT); // was * 4
	data = (USHORT*)malloc( dataSize );
	if ( data == NULL )
	{
		printf("Error in malloc. data is null \n" );
		goto end;
	}
	printf("Soft trig set as PCIE3121_SOFT_TRIGGER_ENABLED | PCIE3121_TRIGGER_SEQUENCE_TRIGGER but seqTriggerCount = 0 \n" );
	hardwareTrigger = PCIE3121_EXT_TRIGGER_DISABLED;

	/* Init timer */
	int iReloadValue = -1;
	do{
		printf("Period for the trigger acquisition (in Milliseconds): ");
		scanf("%d", &iReloadValue);
		
	}while (iReloadValue < 0);

	/* init and start an analog input sequence without DMA */
	returnValue = i_PCIe3121_InitAndStartAnalogInputSequence (boardHandle,
		sequenceNbr,				/* 0: continuous, else, the number of sequence to acquire before stopping */
		sequenceBeforeInterrupt,	/* number of sequence in one interruption */
		8,							/* Number of channels in the sequence */
		channels,
		gain,
		polarity,				
		dwConvertingTime_us,						/* 5us converting time */
		dwDelayTime_us,								/* 5us delay time */
		PCIE3121_SOFT_TRIGGER_ENABLED | PCIE3121_TRIGGER_SINGLE_SEQUENCE,
		PCIE3121_EXT_TRIGGER_DISABLED,
		0 );
	if ( returnValue != 0 )
	{
		printf("Error %d in i_PCIe3121_StartAnalogInputSequence \n", returnValue);
		goto end;
	}
	else
		printf("InitAndStartAnalogInpSeq OK.\n");

	printf("\n Press any key to start the sample ...\n");
	_getch();

		/* trigger the acquisition */
	i_PCIe3121_InitAndStartTimer( boardHandle,
		0, //TimerNb,
		PCIE3121_TIMER_MODE_2, //TimerMode,
		PCIE3121_TIME_BASE_MILLISECONDS, //TimeBase,
		PCIE3121_OUTPUT_DISABLED, //OutputMode,
		true, //InterruptEnabled,
		iReloadValue, //ReloadValue,
		false); //WatchdogEnabled
	if ( returnValue != 0 )
	{
		printf("Error %d in i_PCIe3121_InitAndStartTimer \n", returnValue);
		goto end;
	}
	else
		printf("i_PCIe3121_InitAndStartTimer OK.\n");

	for(;;)
	{
		if (_kbhit())
		{
			hitChar = _getch();
			//ESC
			if ( (hitChar == 27))
			{
				/* exit the loop */
				printf("ESC pressed - stop app\n");
				break;
			}
		}
		else if (errorOccured != 0)
		{
			printf("Error occurred %d - stop app\n", errorOccured);
			break;
		}

		Sleep(100);
		DisplayRunningStatus();
	}


end:
	/* stop the timer */
	returnValue = i_PCIe3121_StopTimer( boardHandle, 0);
	if (returnValue != 0)
		printf("Error %d in i_PCIe3121_StopTimer \n", returnValue);
	else
		printf("i_PCIe3121_StopTimer OK \n");

	/* stop the acquisition */
	returnValue = i_PCIe3121_StopAnalogAcquisition( boardHandle );
	if (returnValue != 0)
		printf("Error %d in i_PCIe3121_StopAnalogAcquisition \n", returnValue);
	else
		printf("i_PCIe3121_StopAnalogAcquisition OK \n");

	/* reset the interrupt routine */
	returnValue = i_PCIe3121_ResetBoardIntRoutine( boardHandle );
	if (returnValue != 0)
		printf("Error %d in i_PCIe3121_ResetBoardIntRoutine \n", returnValue);
	else
		printf("i_PCIe3121_ResetBoardIntRoutine OK \n");

	/* close the board */
	returnValue = i_PCIe3121_CloseBoard(boardHandle);
	if (returnValue != 0)
		printf("Error %d in i_PCIe3121_CloseBoard \n", returnValue);
	else
		printf("i_PCIe3121_CloseBoard OK \n");

	/* free memory */
	free(data);

	/* end */
	printf("\n Press any key to exit the sample ...");
	_getch();

	return 0;
}

