#include "stdafx.h"

int _tmain( int /*argc*/, _TCHAR* /*argv[]*/ )
{
	int		i_ReturnValue;
	HANDLE	h_BoardHandle;
	ULONG	ul_ReadValue;

	UCHAR	uc_TimerMode, uc_TimeBase, uc_EnableInterrupts, uc_OutputMode;
	USHORT	us_ReloadValue, us_TimerValue;

	UCHAR  channels[4] = {0, 1, 2, 3};
	UCHAR  polarity[4] = {0, 0, 0, 0};
	USHORT values[4] = { 0x7FFF, 0x7FFF, 0x7FFF, 0x7FFF };

	i_ReturnValue = OpenBoard(&h_BoardHandle);
	if (i_ReturnValue != 0)
	{
		printf("Error %d opening the board \n", i_ReturnValue);
		goto end;
	}

	// Ask for timer mode
	ReadUlong("Which timer mode do you want to use? (0: Mode 2, 1: Mode 3) ", 0, 1, &ul_ReadValue);
	switch ( ul_ReadValue )
	{
		case 0:		uc_TimerMode = PCIE3121_TIMER_MODE_2;		break;
		case 1:		uc_TimerMode = PCIE3121_TIMER_MODE_3;		break;
		default:	printf("Invalid timer mode selection \n");	goto cleanup;
	}
	
	// Ask for output mode
	ReadUlong("Which output mode do you want to use? (0: Output disabled, 1: Output enabled, 2: Inverted output) ", 0, 2, &ul_ReadValue);
	switch ( ul_ReadValue )
	{
		case 0:		uc_OutputMode = PCIE3121_OUTPUT_DISABLED;	break;
		case 1:		uc_OutputMode = PCIE3121_OUTPUT_ENABLED;	break;
		case 2:		uc_OutputMode = PCIE3121_OUTPUT_INVERTED;	break;
		default:	printf("Invalid output mode \n");			goto cleanup;
	}
	
	// Ask for interrupts
	ReadUlong("Do you want to enable interrupts? (0: No, 1: Yes) ", 0, 1, &ul_ReadValue);
	uc_EnableInterrupts = (UCHAR) ul_ReadValue;

	// Ask for time base
	ReadUlong("Which time base do you want to use? (0: Microseconds, 1: Milliseconds, 2: Seconds) ", 0, 2, &ul_ReadValue);
	switch ( ul_ReadValue )
	{
		case 0:		uc_TimeBase = PCIE3121_TIME_BASE_MICROSECONDS;	break;
		case 1:		uc_TimeBase = PCIE3121_TIME_BASE_MILLISECONDS;	break;			
		case 2:		uc_TimeBase = PCIE3121_TIME_BASE_SECONDS;		break;
		default:	printf("Invalid time base \n");					goto cleanup;
	}

	// Ask for reload value
	ReadUlong("Which reload value do you want to use? (0 to 65535) ", 0, 65535, &ul_ReadValue );
	us_ReloadValue = (USHORT) ul_ReadValue;

	if ( !uc_EnableInterrupts ) goto skip_interrupt;

	// Enable interrupt routine
	i_ReturnValue = i_PCIe3121_SetBoardIntRoutine( h_BoardHandle, InterruptRoutine );
	if ( i_ReturnValue != 0 )
	{
		printf("Error %d in i_PCIe3121_SetBoardIntRoutine \n", i_ReturnValue);
		goto cleanup;
	}
	printf("i_PCIe3121_SetBoardIntRoutine OK \n");


skip_interrupt:

	i_ReturnValue = i_PCIe3121_InitAndStartTimer(h_BoardHandle,
		1,
		uc_TimerMode,
		uc_TimeBase,
		uc_OutputMode,
		uc_EnableInterrupts,
		us_ReloadValue,
		1);

	if ( i_ReturnValue != 0 )
	{
		printf("Error %d in i_PCIe3121_StartTimer1 \n", i_ReturnValue);
		goto cleanup_interrupt;
	}
	printf("i_PCIe3121_StartTimer1 OK \n");

	printf("Press the space key to trigger the timer \n");
	printf("Press the enter key to set the outputs \n");

	printf("\n");

	for(;;)
	{
		i_ReturnValue = i_PCIe3121_ReadTimerValue(h_BoardHandle, 1, &us_TimerValue);
		if ( i_ReturnValue != 0 )
		{
			printf("Error %d in i_PCIe3121_ReadTimer1 \n", i_ReturnValue);
			goto cleanup_timer;
		}

		printf("Timer 1 value is %u \r", us_TimerValue );

		if ( _kbhit() )
		{
			switch( _getch() )
			{
				case 32: // Spacebar key
					i_ReturnValue = i_PCIe3121_TriggerTimer(h_BoardHandle, 1);
					if ( i_ReturnValue != 0 )
					{
						printf("Error %d in i_PCIe3121_TriggerTimer1        \n", i_ReturnValue);
						goto cleanup_timer;
					}
					printf("Timer has been triggered!          \n");
					break;

				case 13: // Enter key
					i_ReturnValue = i_PCIe3121_WriteMoreAnalogOutputs(h_BoardHandle, 4, channels, polarity, values);
					if (i_ReturnValue != 0)
					{
						printf("Error %d in i_PCIe3521_WriteMoreAnalogOutputs        \n", i_ReturnValue);
						goto cleanup_timer;
					}
					printf("Output have been set!          \n");
					break;

				default:
					goto cleanup_timer;
			}
		}

		Sleep(50);
	}


cleanup_timer:
	i_ReturnValue = i_PCIe3121_StopTimer(h_BoardHandle, 1);
	if (i_ReturnValue != 0) printf("Error %d in i_PCIe3121_StopTimer \n", i_ReturnValue);
	else printf("i_PCIe3121_StopTimer0 OK \n");


cleanup_interrupt:
	if ( uc_EnableInterrupts )
	{
		i_ReturnValue = i_PCIe3121_ResetBoardIntRoutine( h_BoardHandle );
		if (i_ReturnValue != 0) printf("Error %d in i_PCIe3121_ResetBoardIntRoutine \n", i_ReturnValue);
		else printf("i_PCIe3121_ResetBoardIntRoutine OK \n");
	}


cleanup:
	i_ReturnValue = i_PCIe3121_CloseBoard(h_BoardHandle);
	if (i_ReturnValue != 0) printf("Error %d in i_PCIe3121_CloseBoard \n", i_ReturnValue);
	else printf("i_PCIe3121_CloseBoard OK \n");


end:
	printf("\n Press any key to exit the sample ...");
	_getch();

	return 0;
}

