using System;
using System.Collections.Generic;
using System.Text;
using System.Runtime.InteropServices;

namespace PCIe3121_NET
{
    /// <summary>
    /// Contains the call to the DLL functions
    /// </summary>
    public class DllImport
    {
        /// <summary>
        /// Definition of the function that must be given to use the interruption of the board
        /// </summary>
        /// <param name="handle"></param>
        /// <param name="fifoStatus"></param>
        /// <param name="source">Source of the interruption. 1: digital input, 2: digital output, 4: timer, 8: counter</param>
        /// <param name="mask"></param>
        /// <param name="size"></param>
        /// <param name="args"></param>
        [UnmanagedFunctionPointer(CallingConvention.Cdecl)]
        public delegate void InterruptFunctionEventHandler(IntPtr handle,
                                                            byte overflow,
                                                            uint source,
                                                            uint mask,
                                                            IntPtr args,
                                                            uint argsSize
                                                            );

        [DllImport("PCIe3121.dll")]
        public static extern short i_PCIe3121_GetNumberOfBoards(ref byte boardNumber );

        [DllImport("PCIe3121.dll")]
        public static extern short i_PCIe3121_OpenBoardViaIndex(byte index, ref IntPtr deviceHandle );

        [DllImport("PCIe3121.dll")]
        public static extern short i_PCIe3121_CloseBoard(IntPtr deviceHandle);

        [DllImport("PCIe3121.dll")]
        public static extern short i_PCIe3121_SetBoardIntRoutine(IntPtr h_DeviceHandle, [MarshalAs(UnmanagedType.FunctionPtr)] InterruptFunctionEventHandler interruptFunction);

        [DllImport("PCIe3121.dll")]
        public static extern short i_PCIe3121_ResetBoardIntRoutine(IntPtr h_DeviceHandle);

        [DllImport("PCIe3121.dll")]
        public static extern short i_PCIe3121_SetDigitalOutputMemoryOn(IntPtr deviceHandle);

        [DllImport("PCIe3121.dll")]
        public static extern short i_PCIe3121_SetDigitalOutputMemoryOff(IntPtr deviceHandle);

        [DllImport("PCIe3121.dll")]
        public static extern short i_PCIe3121_Set1DigitalOutputOn(IntPtr deviceHandle, byte channel);

        [DllImport("PCIe3121.dll")]
        public static extern short i_PCIe3121_Set1DigitalOutputOff(IntPtr deviceHandle, byte channel);

        [DllImport("PCIe3121.dll")]
        public static extern short i_PCIe3121_Set4DigitalOutputsOn(IntPtr deviceHandle, byte value);

        [DllImport("PCIe3121.dll")]
        public static extern short i_PCIe3121_Set4DigitalOutputsOff(IntPtr deviceHandle, byte value);

        [DllImport("PCIe3121.dll")]
        public static extern short i_PCIe3121_Get1DigitalOutputStatus(IntPtr deviceHandle, byte channel, ref byte outputStatus);

        [DllImport("PCIe3121.dll")]
        public static extern short i_PCIe3121_Get4DigitalOutputsStatus(IntPtr deviceHandle, ref byte outputStatus);

        [DllImport("PCIe3121.dll")]
        public static extern short i_PCIe3121_Read1DigitalInput(IntPtr deviceHandle, byte channel, ref byte value);

        [DllImport("PCIe3121.dll")]
        public static extern short i_PCIe3121_Read4DigitalInputs(IntPtr deviceHandle, ref byte value);

        [DllImport("PCIe3121.dll")]
        public static extern short i_PCIe3121_InitAndStartTimer(IntPtr deviceHandle, byte timerNb, byte mode, byte timeBase, byte outputMode, byte interruptEnabled, ushort reloadValue, byte watchdogEnabled);

        [DllImport("PCIe3121.dll")]
        public static extern short i_PCIe3121_StopTimer(IntPtr deviceHandle, byte timerNb);

        [DllImport("PCIe3121.dll")]
        public static extern short i_PCIe3121_ReadTimerStatus(IntPtr deviceHandle, byte timerNb, ref byte status, ref byte softwareTriggerStatus, ref ushort value);

        [DllImport("PCIe3121.dll")]
        public static extern short i_PCIe3121_TriggerTimer(IntPtr deviceHandle, byte timerNb);

        [DllImport("PCIe3121.dll")]
        public static extern short i_PCIe3121_SetAnalogInputMode(IntPtr deviceHandle, byte inputMode);

        [DllImport("PCIe3121.dll")]
        public static extern short i_PCIe3121_GetAnalogInputMode(IntPtr deviceHandle, ref byte inputMode);

        [DllImport("PCIe3121.dll")]
        public static extern short i_PCIe3121_InitAndStartAnalogInputSequence(IntPtr deviceHandle,
            uint sequenceCount, 
            uint sequenceBeforeInterrupt,
            byte channelCount,
            byte[] channelArray,
            byte[] gainArray,
            byte[] polarityArray,
            uint convertingTime,
            uint delayTime,
            byte softTriggerMode,
            byte extTriggerMode,
            uint sequenceTriggerCount);

        [DllImport("PCIe3121.dll")]
        public static extern short i_PCIe3121_StopAnalogAcquisition(IntPtr deviceHandle);

        [DllImport("PCIe3121.dll")]
        public static extern int l_PCIe3121_ReadSequenceData(IntPtr deviceHandle, uint size, ushort[] data);

        [DllImport("PCIe3121.dll")]
        public static extern short i_PCIe3121_ReadMoreAnalogInputs(IntPtr deviceHandle,
            byte channelCount,
            byte[] channelArray,
            byte[] gainArray,
            byte[] polarityArray,
            uint convertingTime,
            byte interruptFlag,
            byte softTriggerMode,
            byte extTriggerMode,
            ushort[] value);

        [DllImport("PCIe3121.dll")]
        public static extern short i_PCIe3121_Read1AnalogInput(IntPtr deviceHandle,
            byte channelIndex,
            byte gain,
            byte polarity,
            uint convertingTime,
            byte interruptFlag,
            byte softTriggerMode,
            byte extTriggerMode,
            ref ushort value);

        [DllImport("PCIe3121.dll")]
        public static extern short i_PCIe3121_InitAndStartAutoRefreshAcquisition(IntPtr deviceHandle,
            byte channelCount,
            byte[] channelArray,
            byte[] gainArray,
            byte[] polarityArray,
            uint convertingTime,
            ref IntPtr value);

        [DllImport("PCIe3121.dll")]
        public static extern short i_PCIe3121_TriggerAnalogAcquisition(IntPtr deviceHandle);

        [DllImport("PCIe3121.dll")]
        public static extern short i_PCIe3121_GetAnalogInputChannelsNbr(IntPtr deviceHandle, ref byte analogInputCount);
		
		[DllImport("PCIe3121.dll")]
        public static extern short i_PCIe3121_Write1AnalogOutput(IntPtr h_DeviceHandle, byte uc_Channel, byte uc_Polarity, ushort us_Value);

		[DllImport("PCIe3121.dll")]
        public static extern short i_PCIe3121_WriteMoreAnalogOutputs(IntPtr h_DeviceHandle, byte uc_ChannelNbr, byte[] puc_Channel, byte[] puc_Polarity, ushort[] pus_Value);
    }
}
