//##protect##"FileHeader"
/** @file i_PCIe3121_mexInitAndStartAnalogInputSequence.c
 *
 *  @authors S. Beuque, C. Guinot, J. Krauth, N. Reibel, J. Rose, L. Sartory, S. Weber
 *  @date    Creation: 01.11.2012
 *  @date    $LastChangedDate$
 *  @version $LastChangedRevision$
 *
 *  @par DESCRIPTION
 *  i_PCIe3121_mexInitAndStartAnalogInputSequence MEX file
 *
 *  @par LICENCE
 *  @verbatim
 *  Copyright (C) 2009 - 2012 ADDI-DATA GmbH
 *
 *  ADDI-DATA GmbH
 *  Airpark Business Center
 *  Airport Boulevard B210
 *  77836 Rheinmnster
 *  Germany
 *  Tel: +49(0)7229/1847-0    |  Fax: +49(0)7229/1847-200
 *  http://www.addi-data-com  |  info@addi-data.com
 *  @endverbatim
 **/
//##protect##"FileHeader"

#include "PCIe3121.h"
#include "mex.h"
#include "matrix.h"

/* i_PCIe3121_mexInitAndStartAnalogInputSequence */
void mexFunction(int nlhs, mxArray *plhs[], int nrhs, const mxArray *prhs[])
{
	/* Declaration of the local variables of the function */
	INT *returnValue = mxCalloc(1, sizeof(INT));
	HANDLE deviceHandle = 0;
	DWORD sequenceCount = 0;
	DWORD sequenceBeforeInterrupt = 0;
	BYTE channelCount = 0;
	DWORD i = 0;
	BYTE channels[16];
	BYTE gain[16];
	BYTE polarity[16];
	DWORD convertingTime = 0;
	BYTE interruptFlag = 0;
	BYTE softTriggerMode = 0;
	BYTE extTriggerMode = 0;
	DWORD sequenceTriggerCount = 0;
//##protect##"i_PCIe3121_mexInitAndStartAnalogInputSequence_VariablesDefinition"
//##protect##"i_PCIe3121_mexInitAndStartAnalogInputSequence_VariablesDefinition"

	/* Check the number of input and output parameters */
	if (nrhs != 12)
		mexErrMsgIdAndTxt("MyToolbox:i_PCIe3121_mexInitAndStartAnalogInputSequence:nrhs", "12 inputs are required.\n");
	if (nlhs != 1)
		mexErrMsgIdAndTxt("MyToolbox:i_PCIe3121_mexInitAndStartAnalogInputSequence:nlhs", "1 output is required.\n");

	/* Acquisition of the input(s) */
	deviceHandle = (HANDLE)(LONG)mxGetScalar(prhs[0]);
	sequenceCount = (DWORD)mxGetScalar(prhs[1]);
	sequenceBeforeInterrupt = (DWORD)mxGetScalar(prhs[2]);
	channelCount = (BYTE)mxGetScalar(prhs[3]);
	for (i = 0; i < 16; i++)
		channels[i] = (BYTE)mxGetPr(prhs[4])[i];
	for (i = 0; i < 16; i++)
		gain[i] = (BYTE)mxGetPr(prhs[5])[i];
	for (i = 0; i < 16; i++)
		polarity[i] = (BYTE)mxGetPr(prhs[6])[i];
	convertingTime = (DWORD)mxGetScalar(prhs[7]);
	interruptFlag = (BYTE)mxGetScalar(prhs[8]);
	softTriggerMode = (BYTE)mxGetScalar(prhs[9]);
	extTriggerMode = (BYTE)mxGetScalar(prhs[10]);
	sequenceTriggerCount = (DWORD)mxGetScalar(prhs[11]);

	/* Check of the values of the input parameters */
	if ((channelCount < 1 || channelCount > 16))
		mexErrMsgIdAndTxt("MyToolbox:i_PCIe3121_mexInitAndStartAnalogInputSequence:nlhs", "The channelCount input must be between 1 and 16.\n");
	if ((convertingTime < 5 || convertingTime > 65535000))
		mexErrMsgIdAndTxt("MyToolbox:i_PCIe3121_mexInitAndStartAnalogInputSequence:nlhs", "The convertingTime input must be between 5 and 65535000.\n");
	if ((interruptFlag < 0 || interruptFlag > 1))
		mexErrMsgIdAndTxt("MyToolbox:i_PCIe3121_mexInitAndStartAnalogInputSequence:nlhs", "The interruptFlag input must be between 0 and 1.\n");
	if ((softTriggerMode < 0 || softTriggerMode > 1))
		mexErrMsgIdAndTxt("MyToolbox:i_PCIe3121_mexInitAndStartAnalogInputSequence:nlhs", "The softTriggerMode input must be between 0 and 1.\n");
	if ((extTriggerMode < 0 || extTriggerMode > 3))
		mexErrMsgIdAndTxt("MyToolbox:i_PCIe3121_mexInitAndStartAnalogInputSequence:nlhs", "The extTriggerMode input must be between 0 and 3.\n");

//##protect##"i_PCIe3121_mexInitAndStartAnalogInputSequence_Start"
//##protect##"i_PCIe3121_mexInitAndStartAnalogInputSequence_Start"

	/* Call of the C function */
	*returnValue = i_PCIe3121_InitAndStartAnalogInputSequence(deviceHandle, sequenceCount, sequenceBeforeInterrupt, channelCount, channels, gain, polarity, convertingTime, interruptFlag, softTriggerMode, extTriggerMode, sequenceTriggerCount);

	/* Creation of the output(s) */
	plhs[0] = mxCreateNumericArray(0, NULL, mxINT16_CLASS, mxREAL);
	mxSetData(plhs[0], returnValue);  mxSetM(plhs[0], 1);  mxSetN(plhs[0], 1);

//##protect##"i_PCIe3121_mexInitAndStartAnalogInputSequence_End"
//##protect##"i_PCIe3121_mexInitAndStartAnalogInputSequence_End"
}
