//##protect##"FileHeader"
/** @file i_PCIe3121_mexInitAndStartAutoRefreshAcquisition.c
 *
 *  @authors S. Beuque, C. Guinot, J. Krauth, N. Reibel, J. Rose, L. Sartory, S. Weber
 *  @date    Creation: 01.11.2012
 *  @date    $LastChangedDate$
 *  @version $LastChangedRevision$
 *
 *  @par DESCRIPTION
 *  i_PCIe3121_mexInitAndStartAutoRefreshAcquisition MEX file
 *
 *  @par LICENCE
 *  @verbatim
 *  Copyright (C) 2009 - 2012 ADDI-DATA GmbH
 *
 *  ADDI-DATA GmbH
 *  Airpark Business Center
 *  Airport Boulevard B210
 *  77836 Rheinmnster
 *  Germany
 *  Tel: +49(0)7229/1847-0    |  Fax: +49(0)7229/1847-200
 *  http://www.addi-data-com  |  info@addi-data.com
 *  @endverbatim
 **/
//##protect##"FileHeader"

#include "PCIe3121.h"
#include "mex.h"
#include "matrix.h"

/* i_PCIe3121_mexInitAndStartAutoRefreshAcquisition */
void mexFunction(int nlhs, mxArray *plhs[], int nrhs, const mxArray *prhs[])
{
	/* Declaration of the local variables of the function */
	INT *returnValue = mxCalloc(1, sizeof(INT));
	HANDLE deviceHandle = 0;
	BYTE channelCount = 0;
	DWORD i = 0;
	BYTE channels[16];
	BYTE gain[16];
	BYTE polarity[16];
	DWORD convertingTime = 0;
	PDWORD *autoRefreshValues = mxCalloc(1, sizeof(PDWORD));
//##protect##"i_PCIe3121_mexInitAndStartAutoRefreshAcquisition_VariablesDefinition"
//##protect##"i_PCIe3121_mexInitAndStartAutoRefreshAcquisition_VariablesDefinition"

	/* Check the number of input and output parameters */
	if (nrhs != 6)
		mexErrMsgIdAndTxt("MyToolbox:i_PCIe3121_mexInitAndStartAutoRefreshAcquisition:nrhs", "6 inputs are required.\n");
	if (nlhs != 2)
		mexErrMsgIdAndTxt("MyToolbox:i_PCIe3121_mexInitAndStartAutoRefreshAcquisition:nlhs", "2 outputs are required.\n");

	/* Acquisition of the input(s) */
	deviceHandle = (HANDLE)(LONG)mxGetScalar(prhs[0]);
	channelCount = (BYTE)mxGetScalar(prhs[1]);
	for (i = 0; i < 16; i++)
		channels[i] = (BYTE)mxGetPr(prhs[2])[i];
	for (i = 0; i < 16; i++)
		gain[i] = (BYTE)mxGetPr(prhs[3])[i];
	for (i = 0; i < 16; i++)
		polarity[i] = (BYTE)mxGetPr(prhs[4])[i];
	convertingTime = (DWORD)mxGetScalar(prhs[5]);

	/* Check of the values of the input parameters */
	if ((channelCount < 0 || channelCount > 16))
		mexErrMsgIdAndTxt("MyToolbox:i_PCIe3121_mexInitAndStartAutoRefreshAcquisition:nlhs", "The channelCount input must be between 0 and 16.\n");
	if ((convertingTime < 5 || convertingTime > 65535000))
		mexErrMsgIdAndTxt("MyToolbox:i_PCIe3121_mexInitAndStartAutoRefreshAcquisition:nlhs", "The convertingTime input must be between 5 and 65535000.\n");

//##protect##"i_PCIe3121_mexInitAndStartAutoRefreshAcquisition_Start"
//##protect##"i_PCIe3121_mexInitAndStartAutoRefreshAcquisition_Start"

	/* Call of the C function */
	*returnValue = i_PCIe3121_InitAndStartAutoRefreshAcquisition(deviceHandle, channelCount, channels, gain, polarity, convertingTime, autoRefreshValues);

	/* Creation of the output(s) */
	plhs[0] = mxCreateNumericArray(0, NULL, mxINT16_CLASS, mxREAL);
	mxSetData(plhs[0], returnValue);  mxSetM(plhs[0], 1);  mxSetN(plhs[0], 1);
	plhs[1] = mxCreateNumericArray(0, NULL, mxUINT32_CLASS, mxREAL);
	mxSetData(plhs[1], autoRefreshValues);  mxSetM(plhs[1], 1);  mxSetN(plhs[1], 1);

//##protect##"i_PCIe3121_mexInitAndStartAutoRefreshAcquisition_End"
//##protect##"i_PCIe3121_mexInitAndStartAutoRefreshAcquisition_End"
}
