function [returnValue, autoRefreshValues] = i_PCIe3121_mexInitAndStartAutoRefreshAcquisition(deviceHandle, channelCount, channels, gain, polarity, convertingTime)
% i_PCIe3121_mexInitAndStartAutoRefreshAcquisition
%
% Starts an auto-refresh acquisition 
% You must call the set analog input mode function before calling this function. 
% Auto-refresh mode is started once, and returns an array of values, each one corresponding to a channel. 
% These values are automatically refreshed after each conversion cycle, so the values in this array are always the latest ones. 
% Gain acts as a multiplier for the input voltage. Possible gain values are :
% 
%   0 : x1 gain, input range is between -10V and 10V (bipolar)
%   1 : x2 gain, input range is between -5V and 5V (bipolar)
%   2 : x5 gain, input range is between -2V and 2V (bipolar)
%   3 : x10 gain, input range is between -1V and 1V (bipolar)
% 
% The converting time stands for the time spent to acquire one channel. The time to acquire a sequence is dw_ConvertingTime * b_ChannelCount 
%
% Input parameters:
%    deviceHandle              "Handle of the board"
%    channelCount              "The number of channels to be acquired"
%    channels                  "An array containing the channels to be acquired"
%    gain                      "An array containing the gains applied to the channels (0 -> x1, 1 -> x2, 2 -> x5, 3 -> x10)"
%    polarity                  "An array containing the polarities of the channels (0 -> unipolar, 1 -> bipolar)"
%    convertingTime            "The length of time allocated to the A/D conversion (in microseconds)"
%
% Output parameters:
%    returnValue             "Value returned by the function"
%    autoRefreshValues         "Pointer to an array where the values acquired from each channel will be written into"
%
%Example:
% // Add an example here

[returnValue, autoRefreshValues] = i_PCIe3121_mexInitAndStartAutoRefreshAcquisition(deviceHandle, channelCount, channels, gain, polarity, convertingTime);
