//##protect##"FileHeader"
/** @file i_PCIe3121_mexWrite1AnalogOutput.c
 *
 *  @authors S. Beuque, C. Guinot, J. Krauth, N. Reibel, J. Rose, L. Sartory, S. Weber
 *  @date    Creation: 01.11.2012
 *  @date    $LastChangedDate$
 *  @version $LastChangedRevision$
 *
 *  @par DESCRIPTION
 *  i_PCIe3121_mexWrite1AnalogOutput MEX file
 *
 *  @par LICENCE
 *  @verbatim
 *  Copyright (C) 2009 - 2012 ADDI-DATA GmbH
 *
 *  ADDI-DATA GmbH
 *  Airpark Business Center
 *  Airport Boulevard B210
 *  77836 Rheinmnster
 *  Germany
 *  Tel: +49(0)7229/1847-0    |  Fax: +49(0)7229/1847-200
 *  http://www.addi-data-com  |  info@addi-data.com
 *  @endverbatim
 **/
//##protect##"FileHeader"

#include "PCIe3121.h"
#include "mex.h"
#include "matrix.h"

/* i_PCIe3121_mexWrite1AnalogOutput */
void mexFunction(int nlhs, mxArray *plhs[], int nrhs, const mxArray *prhs[])
{
	/* Declaration of the local variables of the function */
	INT *returnValue = mxCalloc(1, sizeof(INT));
	HANDLE deviceHandle = 0;
	BYTE channel = 0;
	BYTE polarity = 0;
	WORD value = 0;
//##protect##"i_PCIe3121_mexWrite1AnalogOutput_VariablesDefinition"
//##protect##"i_PCIe3121_mexWrite1AnalogOutput_VariablesDefinition"

	/* Check the number of input and output parameters */
	if (nrhs != 4)
		mexErrMsgIdAndTxt("MyToolbox:i_PCIe3121_mexWrite1AnalogOutput:nrhs", "4 inputs are required.\n");
	if (nlhs != 1)
		mexErrMsgIdAndTxt("MyToolbox:i_PCIe3121_mexWrite1AnalogOutput:nlhs", "1 output is required.\n");

	/* Acquisition of the input(s) */
	deviceHandle = (HANDLE)(LONG)mxGetScalar(prhs[0]);
	channel = (BYTE)mxGetScalar(prhs[1]);
	polarity = (BYTE)mxGetScalar(prhs[2]);
	value = (WORD)mxGetScalar(prhs[3]);

	/* Check of the values of the input parameters */
	if ((channel < 0 || channel > 8))
		mexErrMsgIdAndTxt("MyToolbox:i_PCIe3121_mexWrite1AnalogOutput:nlhs", "The channel input must be between 0 and 8.\n");
	if ((polarity < 0 || polarity > 1))
		mexErrMsgIdAndTxt("MyToolbox:i_PCIe3121_mexWrite1AnalogOutput:nlhs", "The polarity input must be between 0 and 1.\n");

//##protect##"i_PCIe3121_mexWrite1AnalogOutput_Start"
//##protect##"i_PCIe3121_mexWrite1AnalogOutput_Start"

	/* Call of the C function */
	*returnValue = i_PCIe3121_Write1AnalogOutput(deviceHandle, channel, polarity, value);

	/* Creation of the output(s) */
	plhs[0] = mxCreateNumericArray(0, NULL, mxINT16_CLASS, mxREAL);
	mxSetData(plhs[0], returnValue);  mxSetM(plhs[0], 1);  mxSetN(plhs[0], 1);

//##protect##"i_PCIe3121_mexWrite1AnalogOutput_End"
//##protect##"i_PCIe3121_mexWrite1AnalogOutput_End"
}
